@@ -46,18 +46,16 @@ For more detailed info, see https://docs.streamlit.io.
 from streamlit import logger as _logger
 from streamlit import config as _config
 from streamlit.proto.RootContainer_pb2 import RootContainer
-from streamlit.secrets import Secrets, SECRETS_FILE_LOC
+from streamlit.runtime.secrets import Secrets, SECRETS_FILE_LOC
 
 _LOGGER = _logger.get_logger("root")
 
 # Give the package a version.
-import pkg_resources as _pkg_resources
-from typing import NoReturn
+from importlib_metadata import version as _version
 
-# This used to be pkg_resources.require('streamlit') but it would cause
-# pex files to fail. See #394 for more details.
-__version__ = _pkg_resources.get_distribution("streamlit").version
+__version__: str = _version("streamlit")
 
+from typing import Any, Dict, Iterator, List, NoReturn
 import contextlib as _contextlib
 import sys as _sys
 import threading as _threading
@@ -70,11 +68,13 @@ from streamlit import env_util as _env_util
 from streamlit import source_util as _source_util
 from streamlit import string_util as _string_util
 from streamlit.delta_generator import DeltaGenerator as _DeltaGenerator
-from streamlit.script_run_context import add_script_run_ctx as _add_script_run_ctx
-from streamlit.script_run_context import get_script_run_ctx as _get_script_run_ctx
-from streamlit.script_runner import StopException
-from streamlit.script_runner import RerunException as _RerunException
-from streamlit.script_request_queue import RerunData as _RerunData
+from streamlit.runtime.scriptrunner import (
+    add_script_run_ctx as _add_script_run_ctx,
+    get_script_run_ctx as _get_script_run_ctx,
+    StopException,
+    RerunException as _RerunException,
+    RerunData as _RerunData,
+)
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto import ForwardMsg_pb2 as _ForwardMsg_pb2
 
@@ -82,17 +82,19 @@ from streamlit.proto import ForwardMsg_pb2 as _ForwardMsg_pb2
 # syntax pass mypy checking with implicit_reexport disabled.
 
 from streamlit.echo import echo as echo
-from streamlit.legacy_caching import cache as cache
-from streamlit.caching import singleton as experimental_singleton
-from streamlit.caching import memo as experimental_memo
+from streamlit.runtime.legacy_caching import cache as cache
+from streamlit.runtime.caching import (
+    singleton as experimental_singleton,
+    memo as experimental_memo,
+)
 
 # This is set to True inside cli._main_run(), and is False otherwise.
 # If False, we should assume that DeltaGenerator functions are effectively
 # no-ops, and adapt gracefully.
-_is_running_with_streamlit = False
+_is_running_with_streamlit: bool = False
 
 
-def _update_logger():
+def _update_logger() -> None:
     _logger.set_log_level(_config.get_option("logger.level").upper())
     _logger.update_formatter()
     _logger.init_tornado_logs()
@@ -123,6 +125,7 @@ camera_input = _main.camera_input
 checkbox = _main.checkbox
 code = _main.code
 columns = _main.columns
+tabs = _main.tabs
 container = _main.container
 dataframe = _main.dataframe
 date_input = _main.date_input
@@ -155,6 +158,7 @@ radio = _main.radio
 selectbox = _main.selectbox
 select_slider = _main.select_slider
 slider = _main.slider
+snow = _main.snow
 subheader = _main.subheader
 success = _main.success
 table = _main.table
@@ -189,13 +193,15 @@ _arrow_vega_lite_chart = _main._arrow_vega_lite_chart
 
 # Config
 get_option = _config.get_option
-from streamlit.commands.page_config import set_page_config
+from streamlit.commands.page_config import set_page_config as set_page_config
 
 # Session State
 
-from streamlit.state.session_state import LazySessionState
+from streamlit.runtime.state import SessionStateProxy
+from streamlit.user_info import UserInfoProxy
 
-session_state = LazySessionState()
+session_state = SessionStateProxy()
+experimental_user = UserInfoProxy()
 
 
 # Beta APIs
@@ -205,7 +211,7 @@ beta_expander = _main.beta_expander
 beta_columns = _main.beta_columns
 
 
-def set_option(key, value):
+def set_option(key: str, value: Any) -> None:
     """Set config option.
 
     Currently, only the following config options can be set within the script itself:
@@ -227,7 +233,12 @@ def set_option(key, value):
         The new value to assign to this config option.
 
     """
-    opt = _config._config_options_template[key]
+    try:
+        opt = _config._config_options_template[key]
+    except KeyError as ke:
+        raise StreamlitAPIException(
+            "Unrecognized config option: {key}".format(key=key)
+        ) from ke
     if opt.scriptable:
         _config.set_option(key, value)
         return
@@ -239,7 +250,7 @@ def set_option(key, value):
     )
 
 
-def experimental_show(*args):
+def experimental_show(*args: Any) -> None:
     """Write arguments and *argument names* to your app for debugging purposes.
 
     Show() has similar properties to write():
@@ -260,7 +271,7 @@ def experimental_show(*args):
     >>> dataframe = pd.DataFrame({
     ...     'first column': [1, 2, 3, 4],
     ...     'second column': [10, 20, 30, 40],
-    ... }))
+    ... })
     >>> st.experimental_show(dataframe)
 
     Notes
@@ -305,12 +316,21 @@ def experimental_show(*args):
             markdown("**%s**" % escaped)
             write(args[idx])
 
-    except Exception:
+    except Exception as raised_exc:
         _, exc, exc_tb = _sys.exc_info()
+        if exc is None:
+            # Presumably, exc should never be None, but it is typed as
+            # Optional, and I don't know the internals of sys.exc_info() well
+            # enough to just use a cast here. Hence, the runtime check.
+            raise RuntimeError(
+                "Unexpected state: exc was None. If you see this message, "
+                "please create an issue at "
+                "https://github.com/streamlit/streamlit/issues"
+            ) from raised_exc
         exception(exc)
 
 
-def experimental_get_query_params():
+def experimental_get_query_params() -> Dict[str, List[str]]:
     """Return the query parameters that is currently showing in the browser's URL bar.
 
     Returns
@@ -336,11 +356,11 @@ def experimental_get_query_params():
     """
     ctx = _get_script_run_ctx()
     if ctx is None:
-        return ""
+        return {}
     return _parse.parse_qs(ctx.query_string)
 
 
-def experimental_set_query_params(**query_params):
+def experimental_set_query_params(**query_params: Any) -> None:
     """Set the query parameters that are shown in the browser's URL bar.
 
     Parameters
@@ -371,7 +391,7 @@ def experimental_set_query_params(**query_params):
 
 
 @_contextlib.contextmanager
-def spinner(text="In progress..."):
+def spinner(text: str = "In progress...") -> Iterator[None]:
     """Temporarily displays a message while executing a block of code.
 
     Parameters
@@ -387,8 +407,8 @@ def spinner(text="In progress..."):
     >>> st.success('Done!')
 
     """
-    import streamlit.legacy_caching.caching as legacy_caching
-    import streamlit.caching as caching
+    import streamlit.runtime.legacy_caching.caching as legacy_caching
+    import streamlit.runtime.caching as caching
     from streamlit.elements.utils import clean_text
     from streamlit.proto.Spinner_pb2 import Spinner as SpinnerProto
 
@@ -431,7 +451,7 @@ def spinner(text="In progress..."):
                 message.empty()
 
 
-def _transparent_write(*args):
+def _transparent_write(*args: Any) -> Any:
     """This is just st.write, but returns the arguments you passed to it."""
     write(*args)
     if len(args) == 1:
@@ -442,10 +462,10 @@ def _transparent_write(*args):
 # We want to show a warning when the user runs a Streamlit script without
 # 'streamlit run', but we need to make sure the warning appears only once no
 # matter how many times __init__ gets loaded.
-_use_warning_has_been_displayed = False
+_use_warning_has_been_displayed: bool = False
 
 
-def _maybe_print_use_warning():
+def _maybe_print_use_warning() -> None:
     """Print a warning if Streamlit is imported but not being run with `streamlit run`.
     The warning is printed only once.
     """
@@ -491,7 +511,7 @@ def stop() -> NoReturn:
     raise StopException()
 
 
-def experimental_rerun():
+def experimental_rerun() -> NoReturn:
     """Rerun the script immediately.
 
     When `st.experimental_rerun()` is called, the script is halted - no
@@ -503,5 +523,16 @@ def experimental_rerun():
     """
 
     ctx = _get_script_run_ctx()
-    query_string = "" if ctx is None else ctx.query_string
-    raise _RerunException(_RerunData(query_string=query_string))
+
+    query_string = ""
+    page_script_hash = ""
+    if ctx is not None:
+        query_string = ctx.query_string
+        page_script_hash = ctx.page_script_hash
+
+    raise _RerunException(
+        _RerunData(
+            query_string=query_string,
+            page_script_hash=page_script_hash,
+        )
+    )
