@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,21 +14,36 @@
 
 """A Python wrapper around Bokeh."""
 
+import hashlib
 import json
-from typing import cast
+from typing import cast, TYPE_CHECKING
+from typing_extensions import Final
 
-import streamlit
+from streamlit.errors import StreamlitAPIException
 from streamlit.proto.BokehChart_pb2 import BokehChart as BokehChartProto
 
+if TYPE_CHECKING:
+    from bokeh.plotting.figure import Figure
+    from streamlit.delta_generator import DeltaGenerator
+
+ST_BOKEH_VERSION: Final = "2.4.3"
+
 
 class BokehMixin:
-    def bokeh_chart(self, figure, use_container_width=False):
+    def bokeh_chart(
+        self,
+        figure: "Figure",
+        use_container_width: bool = False,
+    ) -> "DeltaGenerator":
         """Display an interactive Bokeh chart.
 
         Bokeh is a charting library for Python. The arguments to this function
         closely follow the ones for Bokeh's `show` function. You can find
         more about Bokeh at https://bokeh.pydata.org.
 
+        To show Bokeh charts in Streamlit, call `st.bokeh_chart`
+        wherever you would call Bokeh's `show`.
+
         Parameters
         ----------
         figure : bokeh.plotting.figure.Figure
@@ -38,9 +53,6 @@ class BokehMixin:
             If True, set the chart width to the column width. This takes
             precedence over Bokeh's native `width` value.
 
-        To show Bokeh charts in Streamlit, call `st.bokeh_chart`
-        wherever you would call Bokeh's `show`.
-
         Example
         -------
         >>> import streamlit as st
@@ -54,26 +66,45 @@ class BokehMixin:
         ...     x_axis_label='x',
         ...     y_axis_label='y')
         ...
-        >>> p.line(x, y, legend='Trend', line_width=2)
+        >>> p.line(x, y, legend_label='Trend', line_width=2)
         >>>
         >>> st.bokeh_chart(p, use_container_width=True)
 
         .. output::
-           https://static.streamlit.io/0.56.0-xTAd/index.html?id=Fdhg51uMbGMLRRxXV6ubzp
-           height: 600px
+           https://doc-bokeh-chart.streamlitapp.com/
+           height: 700px
 
         """
+        import bokeh
+
+        if bokeh.__version__ != ST_BOKEH_VERSION:
+            raise StreamlitAPIException(
+                f"Streamlit only supports Bokeh version {ST_BOKEH_VERSION}, "
+                f"but you have version {bokeh.__version__} installed. Please "
+                f"run `pip install --force-reinstall --no-deps bokeh=="
+                f"{ST_BOKEH_VERSION}` to install the correct version."
+            )
+
+        # Generate element ID from delta path
+        delta_path = self.dg._get_delta_path_str()
+        element_id = hashlib.md5(delta_path.encode()).hexdigest()
+
         bokeh_chart_proto = BokehChartProto()
-        marshall(bokeh_chart_proto, figure, use_container_width)
+        marshall(bokeh_chart_proto, figure, use_container_width, element_id)
         return self.dg._enqueue("bokeh_chart", bokeh_chart_proto)
 
     @property
-    def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
+    def dg(self) -> "DeltaGenerator":
         """Get our DeltaGenerator."""
-        return cast("streamlit.delta_generator.DeltaGenerator", self)
+        return cast("DeltaGenerator", self)
 
 
-def marshall(proto, figure, use_container_width):
+def marshall(
+    proto: BokehChartProto,
+    figure: "Figure",
+    use_container_width: bool,
+    element_id: str,
+) -> None:
     """Construct a Bokeh chart object.
 
     See DeltaGenerator.bokeh_chart for docs.
@@ -83,3 +114,4 @@ def marshall(proto, figure, use_container_width):
     data = json_item(figure)
     proto.figure = json.dumps(data)
     proto.use_container_width = use_container_width
+    proto.element_id = element_id
