@@ -15,21 +15,15 @@
 from streamlit.type_util import Key, to_key
 from typing import cast, overload, List, Optional, Union
 from textwrap import dedent
-
-import sys
-
-if sys.version_info >= (3, 8):
-    from typing import Literal
-else:
-    from typing_extensions import Literal
+from typing_extensions import Literal
 
 import streamlit
 from streamlit import config
 from streamlit.logger import get_logger
 from streamlit.proto.FileUploader_pb2 import FileUploader as FileUploaderProto
-from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
-from streamlit.state.widgets import register_widget
-from streamlit.state.session_state import (
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
+from streamlit.runtime.state import (
+    register_widget,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
@@ -39,7 +33,7 @@ from ..proto.Common_pb2 import (
     FileUploaderState as FileUploaderStateProto,
     UploadedFileInfo as UploadedFileInfoProto,
 )
-from ..uploaded_file_manager import UploadedFile, UploadedFileRec
+from streamlit.runtime.uploaded_file_manager import UploadedFile, UploadedFileRec
 from .utils import check_callback_rules, check_session_state_rules
 
 LOGGER = get_logger(__name__)
@@ -236,7 +230,7 @@ class FileUploaderMixin:
         ...     st.write(bytes_data)
 
         .. output::
-           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.file_uploader.py
+           https://doc-file-uploader.streamlitapp.com/
            height: 375px
 
         """
@@ -291,7 +285,6 @@ class FileUploaderMixin:
         )
         file_uploader_proto.multiple_files = accept_multiple_files
         file_uploader_proto.form_id = current_form_id(self.dg)
-        file_uploader_proto.disabled = disabled
         if help is not None:
             file_uploader_proto.help = dedent(help)
 
@@ -336,7 +329,7 @@ class FileUploaderMixin:
         # FileUploader's widget value is a list of file IDs
         # representing the current set of files that this uploader should
         # know about.
-        widget_value, _ = register_widget(
+        widget_state = register_widget(
             "file_uploader",
             file_uploader_proto,
             user_key=key,
@@ -348,7 +341,11 @@ class FileUploaderMixin:
             ctx=ctx,
         )
 
-        file_uploader_state = serialize_file_uploader(widget_value)
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        file_uploader_proto.disabled = disabled
+
+        file_uploader_state = serialize_file_uploader(widget_state.value)
         uploaded_file_info = file_uploader_state.uploaded_file_info
         if ctx is not None and len(uploaded_file_info) != 0:
             newest_file_id = file_uploader_state.max_file_id
@@ -362,7 +359,7 @@ class FileUploaderMixin:
             )
 
         self.dg._enqueue("file_uploader", file_uploader_proto)
-        return cast(SomeUploadedFiles, widget_value)
+        return widget_state.value
 
     @staticmethod
     def _get_file_recs(
