@@ -1,4 +1,4 @@
-# Copyright 2018-2021 Streamlit Inc.
+# Copyright 2018-2022 Streamlit Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,27 @@
 
 import json
 import urllib.parse
-from typing import cast
+from typing import Any, cast, Dict, List, Set, TYPE_CHECKING, Union
+from typing_extensions import Final, Literal, TypeAlias
 
-import streamlit
-from streamlit import caching
+from streamlit.runtime.legacy_caching import caching
 from streamlit import type_util
 from streamlit.logger import get_logger
 from streamlit.proto.PlotlyChart_pb2 import PlotlyChart as PlotlyChartProto
 
-LOGGER = get_logger(__name__)
+if TYPE_CHECKING:
+    import matplotlib
+    import plotly.graph_objs as go
+    from plotly.basedatatypes import BaseFigure
 
-SHARING_MODES = {
+    from streamlit.delta_generator import DeltaGenerator
+
+
+LOGGER: Final = get_logger(__name__)
+
+SharingMode: TypeAlias = Literal["streamlit", "private", "public", "secret"]
+
+SHARING_MODES: Set[SharingMode] = {
     # This means the plot will be sent to the Streamlit app rather than to
     # Plotly.
     "streamlit",
@@ -37,21 +47,39 @@ SHARING_MODES = {
     "secret",
 }
 
+_AtomicFigureOrData: TypeAlias = Union[
+    "go.Figure",
+    "go.Data",
+]
+FigureOrData: TypeAlias = Union[
+    _AtomicFigureOrData,
+    List[_AtomicFigureOrData],
+    # It is kind of hard to figure out exactly what kind of dict is supported
+    # here, as plotly hasn't embraced typing yet. This version is chosen to
+    # align with the docstring.
+    Dict[str, _AtomicFigureOrData],
+    "BaseFigure",
+    "matplotlib.figure.Figure",
+]
+
 
 class PlotlyMixin:
     def plotly_chart(
         self,
-        figure_or_data,
-        use_container_width=False,
-        sharing="streamlit",
-        **kwargs,
-    ):
+        figure_or_data: FigureOrData,
+        use_container_width: bool = False,
+        sharing: SharingMode = "streamlit",
+        **kwargs: Any,
+    ) -> "DeltaGenerator":
         """Display an interactive Plotly chart.
 
         Plotly is a charting library for Python. The arguments to this function
         closely follow the ones for Plotly's `plot()` function. You can find
         more about Plotly at https://plot.ly/python.
 
+        To show Plotly charts in Streamlit, call `st.plotly_chart` wherever you
+        would call Plotly's `py.plot` or `py.iplot`.
+
         Parameters
         ----------
         figure_or_data : plotly.graph_objs.Figure, plotly.graph_objs.Data,
@@ -67,15 +95,11 @@ class PlotlyMixin:
             Use 'streamlit' to insert the plot and all its dependencies
             directly in the Streamlit app using plotly's offline mode (default).
             Use any other sharing mode to send the chart to Plotly chart studio, which
-            requires an account. See https://plotly.com/chart-studio/ for more information.
+            requires an account. See https://plot.ly/python/chart-studio/ for more information.
 
         **kwargs
             Any argument accepted by Plotly's `plot()` function.
 
-
-        To show Plotly charts in Streamlit, call `st.plotly_chart`
-        wherever you would call Plotly's `py.plot` or `py.iplot`.
-
         Example
         -------
 
@@ -104,7 +128,7 @@ class PlotlyMixin:
         >>> st.plotly_chart(fig, use_container_width=True)
 
         .. output::
-           https://static.streamlit.io/0.56.0-xTAd/index.html?id=TuP96xX8JnsoQeUGAPjkGQ
+           https://doc-plotly-chart.streamlitapp.com/
            height: 400px
 
         """
@@ -119,12 +143,18 @@ class PlotlyMixin:
         return self.dg._enqueue("plotly_chart", plotly_chart_proto)
 
     @property
-    def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
+    def dg(self) -> "DeltaGenerator":
         """Get our DeltaGenerator."""
-        return cast("streamlit.delta_generator.DeltaGenerator", self)
+        return cast("DeltaGenerator", self)
 
 
-def marshall(proto, figure_or_data, use_container_width, sharing, **kwargs):
+def marshall(
+    proto: PlotlyChartProto,
+    figure_or_data: FigureOrData,
+    use_container_width: bool,
+    sharing: SharingMode,
+    **kwargs: Any,
+) -> None:
     """Marshall a proto with a Plotly spec.
 
     See DeltaGenerator.plotly_chart for docs.
@@ -167,10 +197,10 @@ def marshall(proto, figure_or_data, use_container_width, sharing, **kwargs):
 
 
 @caching.cache
-def _plot_to_url_or_load_cached_url(*args, **kwargs):
+def _plot_to_url_or_load_cached_url(*args: Any, **kwargs: Any) -> "go.Figure":
     """Call plotly.plot wrapped in st.cache.
 
-    This is so we don't unecessarily upload data to Plotly's SASS if nothing
+    This is so we don't unnecessarily upload data to Plotly's SASS if nothing
     changed since the previous upload.
     """
     try:
@@ -182,7 +212,7 @@ def _plot_to_url_or_load_cached_url(*args, **kwargs):
     return ply.plot(*args, **kwargs)
 
 
-def _get_embed_url(url):
+def _get_embed_url(url: str) -> str:
     parsed_url = urllib.parse.urlparse(url)
 
     # Plotly's embed URL is the normal URL plus ".embed".
