@@ -18,13 +18,13 @@ from typing import Any, Callable, Optional, cast
 import streamlit
 from streamlit.errors import StreamlitAPIException
 from streamlit.proto.Selectbox_pb2 import Selectbox as SelectboxProto
-from streamlit.script_run_context import ScriptRunContext, get_script_run_ctx
-from streamlit.state.session_state import (
+from streamlit.runtime.scriptrunner import ScriptRunContext, get_script_run_ctx
+from streamlit.runtime.state import (
+    register_widget,
     WidgetArgs,
     WidgetCallback,
     WidgetKwargs,
 )
-from streamlit.state.widgets import register_widget
 from streamlit.type_util import Key, OptionSequence, ensure_indexable, to_key
 from streamlit.util import index_
 from .form import current_form_id
@@ -91,7 +91,7 @@ class SelectboxMixin:
         >>> st.write('You selected:', option)
 
         .. output::
-           https://share.streamlit.io/streamlit/docs/main/python/api-examples-source/widget.selectbox.py
+           https://doc-selectbox.streamlitapp.com/
            height: 320px
 
         """
@@ -146,7 +146,6 @@ class SelectboxMixin:
         selectbox_proto.default = index
         selectbox_proto.options[:] = [str(format_func(option)) for option in opt]
         selectbox_proto.form_id = current_form_id(self.dg)
-        selectbox_proto.disabled = disabled
         if help is not None:
             selectbox_proto.help = dedent(help)
 
@@ -160,7 +159,7 @@ class SelectboxMixin:
                 return 0
             return index_(opt, v)
 
-        current_value, set_frontend_value = register_widget(
+        widget_state = register_widget(
             "selectbox",
             selectbox_proto,
             user_key=key,
@@ -172,12 +171,15 @@ class SelectboxMixin:
             ctx=ctx,
         )
 
-        if set_frontend_value:
-            selectbox_proto.value = serialize_select_box(current_value)
+        # This needs to be done after register_widget because we don't want
+        # the following proto fields to affect a widget's ID.
+        selectbox_proto.disabled = disabled
+        if widget_state.value_changed:
+            selectbox_proto.value = serialize_select_box(widget_state.value)
             selectbox_proto.set_value = True
 
         self.dg._enqueue("selectbox", selectbox_proto)
-        return cast(str, current_value)
+        return widget_state.value
 
     @property
     def dg(self) -> "streamlit.delta_generator.DeltaGenerator":
