@@ -13,31 +13,32 @@
 # limitations under the License.
 
 import textwrap
-from typing import Any, Optional, TYPE_CHECKING
+from typing import Any, cast, Hashable, Optional, TYPE_CHECKING, Union
 
 import streamlit
 from streamlit import type_util
 from streamlit.elements.form import is_in_form
 from streamlit.errors import StreamlitAPIException
-from streamlit.state.session_state import get_session_state
-from streamlit.state.session_state import WidgetCallback
-
+from streamlit.runtime.state import get_session_state, WidgetCallback
 
 if TYPE_CHECKING:
     from streamlit.delta_generator import DeltaGenerator
+    from streamlit.type_util import DataFrameCompatible, SupportsStr
 
 
-def clean_text(text: Any) -> str:
+def clean_text(text: "SupportsStr") -> str:
     """Convert an object to text, dedent it, and strip whitespace."""
     return textwrap.dedent(str(text)).strip()
 
 
-def last_index_for_melted_dataframes(data):
+def last_index_for_melted_dataframes(
+    data: Union["DataFrameCompatible", Any]
+) -> Optional[Hashable]:
     if type_util.is_dataframe_compatible(data):
         data = type_util.convert_anything_to_df(data)
 
         if data.index.size > 0:
-            return data.index[-1]
+            return cast(Hashable, data.index[-1])
 
     return None
 
@@ -56,7 +57,7 @@ def check_callback_rules(
         )
 
 
-_shown_default_value_warning = False
+_shown_default_value_warning: bool = False
 
 
 def check_session_state_rules(
