@@ -14,25 +14,138 @@
 
 """A bunch of useful utilities for dealing with types."""
 
+from __future__ import annotations
+
 import re
-from typing import Any, Optional, Sequence, Tuple, Union, cast
+import types
+from typing import (
+    Any,
+    Iterable,
+    NamedTuple,
+    Optional,
+    Sequence,
+    TypeVar,
+    TYPE_CHECKING,
+    Union,
+    cast,
+    overload,
+    List,
+)
+
+from typing_extensions import (
+    Final,
+    Literal,
+    Protocol,
+    TypeAlias,
+    TypeGuard,
+    get_args,
+)
 
-from pandas import DataFrame, Series, Index
-import numpy as np
 import pyarrow as pa
+from pandas.api.types import infer_dtype
 
 from streamlit import errors
 
-OptionSequence = Union[Sequence[Any], DataFrame, Series, Index, np.ndarray]
-Key = Union[str, int]
+if TYPE_CHECKING:
+    import numpy as np
+    import sympy
+    from pandas import DataFrame, Series, Index
+    from pandas.io.formats.style import Styler
+    from plotly.graph_objs._figure import Figure
+    from pydeck.bindings.deck import Deck  # type: ignore[import]
 
 
-def is_type(obj, fqn_type_pattern):
+# The array value field names are part of the larger set of possible value
+# field names. See the explanation for said set below. The message types
+# associated with these fields are distinguished by storing data in a `data`
+# field in their messages, meaning they need special treatment in certain
+# circumstances. Hence, they need their own, dedicated, sub-type.
+ArrayValueFieldName: TypeAlias = Literal[
+    "double_array_value",
+    "int_array_value",
+    "string_array_value",
+]
+
+# A frozenset containing the allowed values of the ArrayValueFieldName type.
+# Useful for membership checking.
+ARRAY_VALUE_FIELD_NAMES: Final = frozenset(
+    cast(
+        "tuple[ArrayValueFieldName, ...]",
+        # NOTE: get_args is not recursive, so this only works as long as
+        # ArrayValueFieldName remains flat.
+        get_args(ArrayValueFieldName),
+    )
+)
+
+# These are the possible field names that can be set in the `value` oneof-field
+# of the WidgetState message (schema found in .proto/WidgetStates.proto).
+# We need these as a literal type to ensure correspondence with the protobuf
+# schema in certain parts of the python code.
+# TODO(harahu): It would be preferable if this type was automatically derived
+#  from the protobuf schema, rather than manually maintained. Not sure how to
+#  achieve that, though.
+ValueFieldName: TypeAlias = Literal[
+    ArrayValueFieldName,
+    "arrow_value",
+    "bool_value",
+    "bytes_value",
+    "double_value",
+    "file_uploader_state_value",
+    "int_value",
+    "json_value",
+    "string_value",
+    "trigger_value",
+]
+
+OptionSequence: TypeAlias = Union[
+    Sequence[Any],
+    "DataFrame",
+    "Series",
+    "Index",
+    "np.ndarray",
+]
+
+Key: TypeAlias = Union[str, int]
+
+
+T = TypeVar("T")
+
+
+# This should really be a Protocol, but can't be, due to:
+# https://github.com/python/mypy/issues/12933
+# https://github.com/python/mypy/issues/13081
+SupportsStr: TypeAlias = object
+
+
+def is_array_value_field_name(obj: object) -> TypeGuard[ArrayValueFieldName]:
+    return obj in ARRAY_VALUE_FIELD_NAMES
+
+
+@overload
+def is_type(
+    obj: object, fqn_type_pattern: Literal["pydeck.bindings.deck.Deck"]
+) -> TypeGuard[Deck]:
+    ...
+
+
+@overload
+def is_type(
+    obj: object, fqn_type_pattern: Literal["plotly.graph_objs._figure.Figure"]
+) -> TypeGuard[Figure]:
+    ...
+
+
+@overload
+def is_type(obj: object, fqn_type_pattern: Union[str, re.Pattern[str]]) -> bool:
+    ...
+
+
+def is_type(obj: object, fqn_type_pattern: Union[str, re.Pattern[str]]) -> bool:
     """Check type without importing expensive modules.
 
     Parameters
     ----------
-    obj : any
+    obj : object
         The object to type-check.
     fqn_type_pattern : str or regex
         The fully-qualified type string or a regular expression.
@@ -54,25 +167,25 @@ def is_type(obj, fqn_type_pattern):
         return fqn_type_pattern.match(fqn_type) is not None
 
 
-def get_fqn(the_type):
+def get_fqn(the_type: type) -> str:
     """Get module.type_name for a given type."""
     module = the_type.__module__
     name = the_type.__qualname__
     return "%s.%s" % (module, name)
 
 
-def get_fqn_type(obj):
+def get_fqn_type(obj: object) -> str:
     """Get module.type_name for a given object."""
     return get_fqn(type(obj))
 
 
-_PANDAS_DF_TYPE_STR = "pandas.core.frame.DataFrame"
-_PANDAS_INDEX_TYPE_STR = "pandas.core.indexes.base.Index"
-_PANDAS_SERIES_TYPE_STR = "pandas.core.series.Series"
-_PANDAS_STYLER_TYPE_STR = "pandas.io.formats.style.Styler"
-_NUMPY_ARRAY_TYPE_STR = "numpy.ndarray"
+_PANDAS_DF_TYPE_STR: Final = "pandas.core.frame.DataFrame"
+_PANDAS_INDEX_TYPE_STR: Final = "pandas.core.indexes.base.Index"
+_PANDAS_SERIES_TYPE_STR: Final = "pandas.core.series.Series"
+_PANDAS_STYLER_TYPE_STR: Final = "pandas.io.formats.style.Styler"
+_NUMPY_ARRAY_TYPE_STR: Final = "numpy.ndarray"
 
-_DATAFRAME_LIKE_TYPES = (
+_DATAFRAME_LIKE_TYPES: Final[tuple[str, ...]] = (
     _PANDAS_DF_TYPE_STR,
     _PANDAS_INDEX_TYPE_STR,
     _PANDAS_SERIES_TYPE_STR,
@@ -80,55 +193,62 @@ _DATAFRAME_LIKE_TYPES = (
     _NUMPY_ARRAY_TYPE_STR,
 )
 
-_DATAFRAME_COMPATIBLE_TYPES = (
+DataFrameLike: TypeAlias = "Union[DataFrame, Index, Series, Styler]"
+
+_DATAFRAME_COMPATIBLE_TYPES: Final[tuple[type, ...]] = (
     dict,
     list,
     type(None),
-)  # type: Tuple[type, ...]
+)
 
-_BYTES_LIKE_TYPES = (
+_DataFrameCompatible: TypeAlias = Union[dict, list, None]
+DataFrameCompatible: TypeAlias = Union[_DataFrameCompatible, DataFrameLike]
+
+_BYTES_LIKE_TYPES: Final[tuple[type, ...]] = (
     bytes,
     bytearray,
 )
 
+BytesLike: TypeAlias = Union[bytes, bytearray]
 
-def is_dataframe(obj):
+
+def is_dataframe(obj: object) -> TypeGuard[DataFrame]:
     return is_type(obj, _PANDAS_DF_TYPE_STR)
 
 
-def is_dataframe_like(obj):
+def is_dataframe_like(obj: object) -> TypeGuard[DataFrameLike]:
     return any(is_type(obj, t) for t in _DATAFRAME_LIKE_TYPES)
 
 
-def is_dataframe_compatible(obj):
+def is_dataframe_compatible(obj: object) -> TypeGuard[DataFrameCompatible]:
     """True if type that can be passed to convert_anything_to_df."""
     return is_dataframe_like(obj) or type(obj) in _DATAFRAME_COMPATIBLE_TYPES
 
 
-def is_bytes_like(obj: Any) -> bool:
+def is_bytes_like(obj: object) -> TypeGuard[BytesLike]:
     """True if the type is considered bytes-like for the purposes of
     protobuf data marshalling."""
     return isinstance(obj, _BYTES_LIKE_TYPES)
 
 
-def to_bytes(obj: Any) -> bytes:
+def to_bytes(obj: BytesLike) -> bytes:
     """Converts the given object to bytes.
 
     Only types for which `is_bytes_like` is true can be converted; anything
     else will result in an exception.
     """
-    if isinstance(obj, bytes):
-        return obj
-    elif isinstance(obj, bytearray):
+    if isinstance(obj, bytearray):
         return bytes(obj)
+    elif isinstance(obj, bytes):
+        return obj
 
     raise RuntimeError(f"{obj} is not convertible to bytes")
 
 
-_SYMPY_RE = re.compile(r"^sympy.*$")
+_SYMPY_RE: Final = re.compile(r"^sympy.*$")
 
 
-def is_sympy_expession(obj):
+def is_sympy_expession(obj: object) -> TypeGuard[sympy.Expr]:
     """True if input is a SymPy expression."""
     if not is_type(obj, _SYMPY_RE):
         return False
@@ -136,21 +256,20 @@ def is_sympy_expession(obj):
     try:
         import sympy
 
-        if isinstance(obj, sympy.Expr):
-            return True
-    except:
+        return isinstance(obj, sympy.Expr)
+    except ImportError:
         return False
 
 
-_ALTAIR_RE = re.compile(r"^altair\.vegalite\.v\d+\.api\.\w*Chart$")
+_ALTAIR_RE: Final = re.compile(r"^altair\.vegalite\.v\d+\.api\.\w*Chart$")
 
 
-def is_altair_chart(obj):
+def is_altair_chart(obj: object) -> bool:
     """True if input looks like an Altair chart."""
     return is_type(obj, _ALTAIR_RE)
 
 
-def is_keras_model(obj):
+def is_keras_model(obj: object) -> bool:
     """True if input looks like a Keras model."""
     return (
         is_type(obj, "keras.engine.sequential.Sequential")
@@ -160,7 +279,7 @@ def is_keras_model(obj):
     )
 
 
-def is_plotly_chart(obj):
+def is_plotly_chart(obj: object) -> TypeGuard[Union[Figure, list[Any], dict[str, Any]]]:
     """True if input looks like a Plotly chart."""
     return (
         is_type(obj, "plotly.graph_objs._figure.Figure")
@@ -169,7 +288,7 @@ def is_plotly_chart(obj):
     )
 
 
-def is_graphviz_chart(obj):
+def is_graphviz_chart(obj: object) -> bool:
     """True if input looks like a GraphViz chart."""
     return (
         # GraphViz < 0.18
@@ -181,21 +300,21 @@ def is_graphviz_chart(obj):
     )
 
 
-def _is_plotly_obj(obj):
+def _is_plotly_obj(obj: object) -> bool:
     """True if input if from a type that lives in plotly.plotly_objs."""
     the_type = type(obj)
     return the_type.__module__.startswith("plotly.graph_objs")
 
 
-def _is_list_of_plotly_objs(obj):
-    if type(obj) is not list:
+def _is_list_of_plotly_objs(obj: object) -> TypeGuard[list[Any]]:
+    if not isinstance(obj, list):
         return False
     if len(obj) == 0:
         return False
     return all(_is_plotly_obj(item) for item in obj)
 
 
-def _is_probably_plotly_dict(obj):
+def _is_probably_plotly_dict(obj: object) -> TypeGuard[dict[str, Any]]:
     if not isinstance(obj, dict):
         return False
 
@@ -214,15 +333,12 @@ def _is_probably_plotly_dict(obj):
     return False
 
 
-_FUNCTION_TYPE = type(lambda: 0)
-
-
-def is_function(x):
+def is_function(x: object) -> TypeGuard[types.FunctionType]:
     """Return True if x is a function."""
-    return type(x) == _FUNCTION_TYPE
+    return isinstance(x, types.FunctionType)
 
 
-def is_namedtuple(x):
+def is_namedtuple(x: object) -> TypeGuard[NamedTuple]:
     t = type(x)
     b = t.__bases__
     if len(b) != 1 or b[0] != tuple:
@@ -233,15 +349,26 @@ def is_namedtuple(x):
     return all(type(n).__name__ == "str" for n in f)
 
 
-def is_pandas_styler(obj):
+def is_pandas_styler(obj: object) -> TypeGuard[Styler]:
     return is_type(obj, _PANDAS_STYLER_TYPE_STR)
 
 
-def is_pydeck(obj):
+def is_pydeck(obj: object) -> TypeGuard[Deck]:
     """True if input looks like a pydeck chart."""
     return is_type(obj, "pydeck.bindings.deck.Deck")
 
 
+def is_sequence(seq: Any) -> bool:
+    """True if input looks like a sequence."""
+    if isinstance(seq, str):
+        return False
+    try:
+        len(seq)
+    except Exception:
+        return False
+    return True
+
+
 def convert_anything_to_df(df: Any) -> DataFrame:
     """Try to convert different formats to a Pandas Dataframe.
 
@@ -254,7 +381,7 @@ def convert_anything_to_df(df: Any) -> DataFrame:
     pandas.DataFrame
 
     """
-    # This is inefficent as the data will be converted back to Arrow
+    # This is inefficient as the data will be converted back to Arrow
     # when marshalled to protobuf, but area/bar/line charts need
     # DataFrame magic to generate the correct output.
     if isinstance(df, pa.Table):
@@ -292,7 +419,17 @@ Offending object:
         )
 
 
-def ensure_iterable(obj):
+@overload
+def ensure_iterable(obj: Iterable[T]) -> Iterable[T]:
+    ...
+
+
+@overload
+def ensure_iterable(obj: DataFrame) -> Iterable[Any]:
+    ...
+
+
+def ensure_iterable(obj: Union[DataFrame, Iterable[T]]) -> Iterable[Any]:
     """Try to convert different formats to something iterable. Most inputs
     are assumed to be iterable, but if we have a DataFrame, we can just
     select the first column to iterate over. If the input is not iterable,
@@ -308,15 +445,25 @@ def ensure_iterable(obj):
 
     """
     if is_dataframe(obj):
-        return obj.iloc[:, 0]
+        return cast(Iterable[Any], obj.iloc[:, 0])
 
     try:
         iter(obj)
-        return obj
-    except:
+        return cast(Iterable[T], obj)
+    except TypeError:
         raise
 
 
+@overload
+def ensure_indexable(obj: Sequence[T]) -> Sequence[T]:
+    ...
+
+
+@overload
+def ensure_indexable(obj: OptionSequence) -> Sequence[Any]:
+    ...
+
+
 def ensure_indexable(obj: OptionSequence) -> Sequence[Any]:
     """Try to ensure a value is an indexable Sequence. If the collection already
     is one, it has the index method that we need. Otherwise, convert it to a list.
@@ -366,6 +513,16 @@ def pyarrow_table_to_bytes(table: pa.Table) -> bytes:
     return cast(bytes, sink.getvalue().to_pybytes())
 
 
+def convert_mixed_columns_to_string(
+    df: DataFrame, selected_columns: Optional[List[str]] = None
+) -> DataFrame:
+    for col in selected_columns or df.columns:
+        column = df[col]
+        if column.dtype == "object" and "mixed" in infer_dtype(column):
+            df[col] = column.astype(str)
+    return df
+
+
 def data_frame_to_bytes(df: DataFrame) -> bytes:
     """Serialize pandas.DataFrame to bytes using Apache Arrow.
 
@@ -376,7 +533,12 @@ def data_frame_to_bytes(df: DataFrame) -> bytes:
 
     """
     try:
-        table = pa.Table.from_pandas(df)
+        try:
+            table = pa.Table.from_pandas(df)
+        except pa.ArrowTypeError:
+            # Fallback: Convert all object types to string
+            df = convert_mixed_columns_to_string(df)
+            table = pa.Table.from_pandas(df)
         return pyarrow_table_to_bytes(table)
     except Exception as e:
         _NUMPY_DTYPE_ERROR_MESSAGE = "Could not convert dtype"
@@ -406,6 +568,16 @@ def bytes_to_data_frame(source: bytes) -> DataFrame:
     return reader.read_pandas()
 
 
+@overload
+def to_key(key: None) -> None:
+    ...
+
+
+@overload
+def to_key(key: Key) -> str:
+    ...
+
+
 def to_key(key: Optional[Key]) -> Optional[str]:
     if key is None:
         return None
